// Control.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "Control.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "Sequence.h"
#include "ParamList.h"
#include "MainDialog.h"
#include "ParamList.h"
#include "IOList.h"
#include "Output.h"
#include "UniMessList.h"
#include "MeasurementQueue.h"
#include "GPIBDevice.h"
#include "SerialDevice.h"
#include "Serial.h"
#include "SlaveServer.h"
#include "SystemParamList.h"
#include "SlaveIOList.h"
#include "SlaveMainDialog.h"
#include "UtilityDialog.h"
//#include "NetIO.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CControlApp

BEGIN_MESSAGE_MAP(CControlApp, CWinApp)
	//{{AFX_MSG_MAP(CControlApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

COutput *Output;
//CNetIO *NetIO;
CIOList IOList;
CParamList ParamList;
CSystemParamList SystemParamList;
CSlaveIOList SlaveIOList;
CSequence Sequence;
CSlaveServer SlaveServer;
CUniMessList UniMessList;
CMeasurementQueue MeasurementQueue(false);
CMeasurementQueue ReferenceMeasurementQueue(true);
CUtilityDialog UtilityDialog;
CDialog* ActiveDialog;
bool HardwareAccess;

/////////////////////////////////////////////////////////////////////////////
// CControlApp construction

CControlApp::CControlApp()
{
	ParamFileName="D:\\ControlGerhard\\ControlParam.dat";
	HardwareAccess=false;
	Output=NULL;		
	ActiveDialog=NULL;
}

CControlApp::~CControlApp() {	
	if (Mutex) {
		SaveParams(ParamFileName);
		ReleaseMutex(Mutex);
	}
//	CWnd::KillTimer(ID_TIMER); 
	if (Output) delete Output;
//	if (NetIO) delete NetIO;
}

void PumpMessages() {
	MSG msg;
	while ( ::PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ) 
	{ 
		if ( !AfxGetApp()->PumpMessage( ) ) 
		{ 	       
	       ::PostQuitMessage(0); 
		    break; 
		} 
	} 
	// let MFC do its idle processing
	LONG lIdle = 0;
	//while ( 
		AfxGetApp()->OnIdle(lIdle++ );// )	;  
	// Perform some background processing here 
	// using another call to OnIdle	
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CControlApp object

CControlApp ControlApp;

/////////////////////////////////////////////////////////////////////////////
// CControlApp initialization

#define MAX_LOADSTRING 100
static TCHAR szMutexName[MAX_LOADSTRING];              // Mutex name

/////////////////////////////////////////////////////////////////////////////
// Creates a mutex only for one application
//    This function tries to create a mutex, but if specified mutex exits
//    this function returns NULL.
HANDLE CreateOneAppMutex(LPCTSTR lpName)
{
    HANDLE hMutex;
    hMutex = CreateMutex(NULL, TRUE, lpName);   // Create mutex
    switch(GetLastError())
    {
    case ERROR_SUCCESS:
        // Mutex created successfully. There is no instances running
        break;
    case ERROR_ALREADY_EXISTS:
        // Mutex already exists so there is a running instance of our app.
        hMutex = NULL;
        break;
    default:
        // Failed to create mutex by unknown reason
        break;
    }
    return hMutex;
}
/*
void CALLBACK EXPORT TimerProc(
   HWND hWnd,      // handle of CWnd that called SetTimer
   UINT nMsg,      // WM_TIMER
   UINT nIDEvent,   // timer identification
   DWORD dwTime    // system time
   ) 
{
	Sequence.Idle();
}*/

BOOL CControlApp::InitInstance()
{
	LoadString(m_hInstance, IDC_MUTEX, szMutexName, MAX_LOADSTRING);
	Mutex=CreateOneAppMutex(szMutexName);
	if (Mutex==NULL) {
		//another copy of Control is already running
		//->disable hardware access
		AfxMessageBox("CControlApp::InitInstance: Another instance of Control is running. Disabling hardware access. ");
		HardwareAccess=false;
	} else HardwareAccess=false; //needs to be true to work	
	//HardwareAccess=false;
	//bool IAMMASTER=true;
	//HardwareAccess=false;

	Output=new COutput(HardwareAccess);
	GPIB.SetHardwareAccess(HardwareAccess);
	Serial.SetHardwareAccess(false);//HardwareAccess);	
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

//#ifdef _AFXDLL
//	Enable3dControls();			// Call this when using MFC in a shared DLL
//#else
//	Enable3dControlsStatic();	// Call this when linking to MFC statically
//#endif
	LoadParams(ParamFileName);
	LoadReferenceParams(*ParameterReferenceFileName);	

//	if (IAMMASTER) ComputerNumber=0;
//  else ComputerNumber=1;

	if (!AfxSocketInit()) {
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}
	if (ComputerNumber<0) HardwareAccess=false;	
	Sequence.SetHardwareAccess(HardwareAccess);
	Sequence.ConfigureHardware();

	if (ComputerNumber==-1) {
		SlaveServer.Disable();
		UniMessList.ConstructUniMessList();
		CEasyDialog *dlg=new CMainDialog((HardwareAccess) ? "Slave Experiment Control" : "Slave Remote Control",HardwareAccess);		
		ActiveDialog=dlg;
		m_pMainWnd = dlg;
		dlg->DoModal();
		//delete dlg;
		ActiveDialog=NULL;
	} else if (ComputerNumber==0) {
		SlaveServer.Disable();
		IOList.InitializeOutputs();
		Sequence.DefineCalibrations();
		Sequence.InitializeSystemFirstTime();	
		UniMessList.ConstructUniMessList();	
		CEasyDialog *dlg=new CMainDialog((HardwareAccess) ? "Experiment Control" : "Remote Control",HardwareAccess);
		ActiveDialog=dlg;
		m_pMainWnd = dlg;	
		dlg->DoModal();
		Sequence.ShutDown();
		delete dlg;
		ActiveDialog=NULL;
	} else {
		SlaveIOList.SlaveListRegisterOutputs(Output->GetNrDigitalOut(),Output->GetNrAnalogOut());
		SlaveIOList.InitializeOutputs();
		CEasyDialog *dlg=new CSlaveMainDialog();
		ActiveDialog=dlg;
		m_pMainWnd = dlg;

		DebugSlaveServerOn=true;

		SlaveServer.DebugStart();
		SlaveServer.StartNetworkServer(dlg,false);
		dlg->DoModal();
		SlaveServer.StopNetworkServer(false);
		delete dlg;
		ActiveDialog=NULL;
	}	
	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CControlApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CControlApp message handlers


void CControlApp::SaveParameterDialog()
{
	CFileDialog FileDialog( false, "dat", *UserParameterFileName);
	if (FileDialog.DoModal()==IDOK) {		
		*UserParameterFileName=FileDialog.GetPathName();
		SaveParams(*UserParameterFileName);
	}
}

void CControlApp::SaveParams(CString Name) {
	CFile f;
	if (Name=="") Name=ParamFileName;
	if( !f.Open( Name, CFile::modeCreate | CFile::modeWrite ) ) {
		AfxMessageBox("CControlApp::SaveParameterDialog : Unable to open file "+Name);
	} else {
		SaveParams(f);
		f.Close();
	}	
}

void CControlApp::LoadParameterDialog()
{
	CFileDialog FileDialog( true, "dat", *UserParameterFileName, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST);
	if (FileDialog.DoModal()==IDOK) {
		*UserParameterFileName=FileDialog.GetPathName();
		LoadParams(*UserParameterFileName);
	}
}

void CControlApp::LoadParams(CString Name) {		
	CFile f;	
	if (Name=="") Name=ParamFileName;
	if( !f.Open( Name, CFile::modeRead ) ) {
		AfxMessageBox("CControlApp::LoadParameterDialog : Unable to open file\n"+Name+"\nA new parameter file will be created.\nIf this is the first time you start control on this computer, this is not an error.");		
	} else {
		LoadParams(f);
		f.Close();
	}	
}

void CControlApp::LoadReferenceParams(CString Name) {		
	CFile f;	
	if (Name=="") Name=ParamFileName;
	if( !f.Open( Name, CFile::modeRead ) ) {
		AfxMessageBox("CControlApp::LoadReferenceParams : Unable to open file\n"+Name);		
	} else {
		ControlApp.LoadParams(f);
		f.Close();
	}
	ParamList.SwapWithReference();
	UtilityDialog.SwapWithReference();
	SystemParamList.SwapWithReference();	
	LoadParams(ParamFileName);
}

void CControlApp::LoadParams(CFile &f)
{	
	CArchive ar( &f, CArchive::load);
	ParamList.SerializeParams(ar);	
	UtilityDialog.SerializeParams(ar);
	SystemParamList.SerializeParams(ar);
	UniMessList.Serialize(ar);
	ar.Close();	
}

void CControlApp::SaveParams(CFile &f)
{	
	CArchive ar( &f, CArchive::store);
	ParamList.SerializeParams(ar);
	UtilityDialog.SerializeParams(ar);
	SystemParamList.SerializeParams(ar);
	UniMessList.Serialize(ar);
	ar.Close();	
}

BOOL CControlApp::OnIdle(LONG lCount) 
{	
	CWinApp::OnIdle(lCount);
	Sequence.Idle(ActiveDialog);
	Sleep(1);
	return true;
}


void CControlApp::OnWakeUp()
{
	Sequence.WakeUp();
}

void CControlApp::PauseSystemDuringMeasurment(CWnd* parent) {
	Sequence.PauseSystemDuringMeasurment(parent);
}